@@ -53,7 +53,6 @@ disambiguation, and in some cases warns.
 
 from __future__ import absolute_import, print_function, unicode_literals
 import os
-import sys
 import gzip
 import dateutil.parser  # For parsing date strings
 from distutils.version import LooseVersion
@@ -115,14 +114,14 @@ def _get_git_version():
         cmd = 'git --version'
         value, version, _ = run_shell_command(cmd, shell=True)
         if value != 0:
-            raise VcsError("git --version returned %s, maybe git is not installed"%(value))
+            raise VcsError("git --version returned %s, maybe git is not installed" % (value))
         prefix = 'git version '
         if version is not None and version.startswith(prefix):
             version = version[len(prefix):].strip()
         else:
-            raise VcsError("git --version returned invalid string: '%s'"%version)
+            raise VcsError("git --version returned invalid string: '%s'" % version)
     except VcsError as exc:
-        raise VcsError("Could not determine whether git is installed: %s"%exc)
+        raise VcsError("Could not determine whether git is installed: %s" % exc)
     return version
 
 
@@ -139,9 +138,9 @@ class GitClient(VcsClientBase):
         metadict = {}
         try:
             version = _get_git_version()
-            resetkeep =  LooseVersion(version) >= LooseVersion('1.7.1')
+            resetkeep = LooseVersion(version) >= LooseVersion('1.7.1')
             submodules = LooseVersion(version) > LooseVersion('1.7')
-            metadict["features"] = "'reset --keep': %s, submodules: %s"%(resetkeep, submodules)
+            metadict["features"] = "'reset --keep': %s, submodules: %s" % (resetkeep, submodules)
         except VcsError:
             version = "No git installed"
         metadict["version"] = version
@@ -157,27 +156,33 @@ class GitClient(VcsClientBase):
             return output.rstrip()
         return None
 
-    def detect_presence(self):
+    @staticmethod
+    def static_detect_presence(path):
         # There is a proposed implementation of detect_presence which might be
         # more future proof, but would depend on parsing the output of git
         # See: https://github.com/vcstools/vcstools/pull/10
-        return self.path_exists() and os.path.exists(os.path.join(self._path, '.git'))
+        return os.path.exists(os.path.join(path, '.git'))
 
-    def checkout(self, url, refname=None, verbose=False, shallow=False):
-        """calls git clone and then, if refname was given, update(refname)"""
+    def checkout(self, url, version=None, verbose=False, shallow=False, timeout=None):
+        """calls git clone and then, if version was given, update(version)"""
         if url is None or url.strip() == '':
             raise ValueError('Invalid empty url : "%s"' % url)
 
-        #since we cannot know whether refname names a branch, clone master initially
+        # since we cannot know whether version names a branch, clone master initially
         cmd = 'git clone'
         if shallow:
             cmd += ' --depth 1'
             if LooseVersion(self.gitversion) >= LooseVersion('1.7.10'):
                 cmd += ' --no-single-branch'
-        cmd += ' --recursive %s %s' % (url, self._path)
+        if version is None:
+            # quicker than using _do_update, but undesired when switching branches next
+            cmd += ' --recursive'
+        cmd += ' %s %s' % (url, self._path)
         value, _, msg = run_shell_command(cmd,
                                           shell=True,
+                                          no_filter=True,
                                           show_stdout=verbose,
+                                          timeout=timeout,
                                           verbose=verbose)
         if value != 0:
             if msg:
@@ -187,17 +192,18 @@ class GitClient(VcsClientBase):
         try:
             # update to make sure we are on the right branch. Do not
             # check for "master" here, as default branch could be anything
-            if refname is not None:
-                return self._do_update(refname,
+            if version is not None:
+                return self._do_update(version,
                                        verbose=verbose,
                                        fast_foward=True,
-                                       update_submodules=False)
+                                       timeout=timeout,
+                                       update_submodules=True)
             else:
                 return True
         except GitError:
             return False
 
-    def update_submodules(self, verbose=False):
+    def update_submodules(self, verbose=False, timeout=None):
 
         # update and or init submodules too
         if LooseVersion(self.gitversion) > LooseVersion('1.7'):
@@ -206,21 +212,22 @@ class GitClient(VcsClientBase):
                                             shell=True,
                                             cwd=self._path,
                                             show_stdout=True,
+                                            timeout=timeout,
                                             verbose=verbose)
             if value != 0:
                 return False
         return True
 
-    def update(self, refname=None, verbose=False, force_fetch=False):
+    def update(self, version=None, verbose=False, force_fetch=False, timeout=None):
         """
-        if refname is None, attempts fast-forwarding current branch, if any.
+        if version is None, attempts fast-forwarding current branch, if any.
 
-        Else interprets refname as a local branch, remote branch, tagname,
+        Else interprets version as a local branch, remote branch, tagname,
         hash, etc.
 
         If it is a branch, attempts to move to it unless
         already on it, and to fast-forward, unless not a tracking
-        branch. Else go untracked on tag or whatever refname is. Does
+        branch. Else go untracked on tag or whatever version is. Does
         not leave if current commit would become dangling.
 
         :return: True if already up-to-date with remote or after successful fast_foward
@@ -231,7 +238,7 @@ class GitClient(VcsClientBase):
         try:
             # fetch in any case to get updated tags even if we don't need them
             self._do_fetch()
-            return self._do_update(refname=refname, verbose=verbose)
+            return self._do_update(refname=version, verbose=verbose, timeout=timeout)
         except GitError:
             return False
 
@@ -239,6 +246,7 @@ class GitClient(VcsClientBase):
                    refname=None,
                    verbose=False,
                    fast_foward=True,
+                   timeout=None,
                    update_submodules=True):
         '''
         updates without fetching, thus any necessary fetching must be done before
@@ -269,7 +277,8 @@ class GitClient(VcsClientBase):
 
         if not refname:
             # we are neither tracking, nor did we get any refname to update to
-            return (not update_submodules) or self.update_submodules(verbose=verbose)
+            return (not update_submodules) or self.update_submodules(verbose=verbose,
+                                                                     timeout=timeout)
 
         if same_branch:
             if fast_foward:
@@ -297,7 +306,8 @@ class GitClient(VcsClientBase):
             if not refname_is_branch:
                 current_version = self.get_version()
                 if current_version == refname:
-                    return (not update_submodules) or self.update_submodules(verbose=verbose)
+                    return (not update_submodules) or self.update_submodules(verbose=verbose,
+                                                                             timeout=timeout)
 
             if current_branch is None:
                 if not current_version:
@@ -322,7 +332,7 @@ class GitClient(VcsClientBase):
                                                      fetch=False,
                                                      verbose=verbose):
                             return False
-        return (not update_submodules) or self.update_submodules(verbose=verbose)
+        return (not update_submodules) or self.update_submodules(verbose=verbose, timeout=timeout)
 
     def get_version(self, spec=None):
         """
@@ -336,17 +346,15 @@ class GitClient(VcsClientBase):
         if self.detect_presence():
             command = "git log -1"
             if spec is not None:
-                command += " %s"%sanitized(spec)
+                command += " %s" % sanitized(spec)
             command += " --format='%H'"
-            repeated = False
             output = ''
-            #we repeat the call once after fetching if necessary
+            # we repeat the call once after fetching if necessary
             for _ in range(2):
                 _, output, _ = run_shell_command(command,
                                                  shell=True,
                                                  cwd=self._path)
-                if (output != ''
-                    or spec is None):
+                if (output != '' or spec is None):
                     break
                 # we try again after fetching if given spec had not been found
                 try:
@@ -367,7 +375,8 @@ class GitClient(VcsClientBase):
             # git needs special treatment as it only works from inside
             # use HEAD to also show staged changes. Maybe should be option?
             # injection should be impossible using relpath, but to be sure, we check
-            cmd = "git diff HEAD --src-prefix=%s/ --dst-prefix=%s/ ."%(sanitized(rel_path), sanitized(rel_path))
+            cmd = "git diff HEAD --src-prefix=%s/ --dst-prefix=%s/ ." % \
+                  (sanitized(rel_path), sanitized(rel_path))
             _, response, _ = run_shell_command(cmd, shell=True, cwd=self._path)
             if LooseVersion(self.gitversion) > LooseVersion('1.7'):
                 cmd = 'git submodule foreach --recursive git diff HEAD'
@@ -378,7 +387,7 @@ class GitClient(VcsClientBase):
     def get_log(self, relpath=None, limit=None):
         response = []
 
-        if relpath == None:
+        if relpath is None:
             relpath = ''
 
         if self.path_exists() and os.path.exists(os.path.join(self._path, relpath)):
@@ -388,7 +397,8 @@ class GitClient(VcsClientBase):
             GIT_COMMIT_FIELDS = ['id', 'author', 'email', 'date', 'message']
             GIT_LOG_FORMAT = '%x1f'.join(['%H', '%an', '%ae', '%ad', '%s']) + '%x1e'
 
-            command = "git --work-tree=%s log --format=\"%s\" %s %s " % (self._path, GIT_LOG_FORMAT, limit_cmd, sanitized(relpath))
+            command = "git --work-tree=%s log --format=\"%s\" %s %s " % (self._path, GIT_LOG_FORMAT,
+                                                                         limit_cmd, sanitized(relpath))
             return_code, response_str, stderr = run_shell_command(command, shell=True, cwd=self._path)
 
             if return_code == 0:
@@ -421,9 +431,9 @@ class GitClient(VcsClientBase):
             for line in response.split('\n'):
                 if len(line.strip()) > 0:
                     # prepend relative path
-                    response_processed += '%s%s/%s\n'%(line[0:3],
-                                                       rel_path,
-                                                       line[3:])
+                    response_processed += '%s%s/%s\n' % (line[0:3],
+                                                         rel_path,
+                                                         line[3:])
             if LooseVersion(self.gitversion) > LooseVersion('1.7'):
                 command = "git submodule foreach --recursive git status -s"
                 if not untracked:
@@ -436,7 +446,7 @@ class GitClient(VcsClientBase):
                         continue
                     if len(line.strip()) > 0:
                         # prepend relative path
-                        response_processed+=line[0:3]+rel_path+'/'+line[3:]+'\n'
+                        response_processed += line[0:3] + rel_path + '/' + line[3:] + '\n'
             response = response_processed
         return response
 
@@ -461,7 +471,6 @@ class GitClient(VcsClientBase):
                     return True
         return False
 
-
     def is_local_branch(self, branch_name):
         if self.path_exists():
             _, output, _ = run_shell_command('git branch',
@@ -499,7 +508,7 @@ class GitClient(VcsClientBase):
             branchname = current_branch or self.get_branch()
             if branchname is None:
                 return None
-            cmd = 'git config --get-all %s'%sanitized('branch.%s.merge'%branchname)
+            cmd = 'git config --get %s' % sanitized('branch.%s.merge' % branchname)
 
             _, output, _ = run_shell_command(cmd,
                                              shell=True,
@@ -508,13 +517,14 @@ class GitClient(VcsClientBase):
                 return None
             lines = output.splitlines()
             if len(lines) > 1:
-                print("vcstools unable to handle multiple merge references for branch %s:\n%s"%(branchname, output))
+                print("vcstools unable to handle multiple merge references for branch %s:\n%s" % (branchname, output))
                 return None
             # get name of configured remote
-            cmd = 'git config --get-all "branch.%s.remote"'%branchname
+            cmd = 'git config --get "branch.%s.remote"' % branchname
             _, output2, _ = run_shell_command(cmd, shell=True, cwd=self._path)
             if output2 != "origin":
-                print("vcstools only handles branches tracking remote 'origin', branch '%s' tracks remote '%s'"%(branchname, output2))
+                print("vcstools only handles branches tracking remote 'origin'," +
+                      " branch '%s' tracks remote '%s'" % (branchname, output2))
                 return None
             output = lines[0]
             # output is either refname, or /refs/heads/refname, or
@@ -551,7 +561,7 @@ class GitClient(VcsClientBase):
         if not tag_name:
             raise ValueError('is_tag requires tag_name, got: "%s"' % tag_name)
         if self.path_exists():
-            cmd = 'git tag -l %s'%sanitized(tag_name)
+            cmd = 'git tag -l %s' % sanitized(tag_name)
             _, output, _ = run_shell_command(cmd, shell=True, cwd=self._path)
             lines = output.splitlines()
             if len(lines) == 1:
@@ -579,12 +589,10 @@ class GitClient(VcsClientBase):
         # not sure what's more performant
         if fetch:
             self._do_fetch()
-        if (refname is not None and
-            refname != '' and
-            version is not None and
-            version!=''):
+        if (refname is not None and refname != '' and
+                version is not None and version != ''):
 
-            cmd = 'git rev-list %s ^%s --parents'%(sanitized(refname), sanitized(version))
+            cmd = 'git rev-list %s ^%s --parents' % (sanitized(refname), sanitized(version))
             _, output, _ = run_shell_command(cmd, shell=True, cwd=self._path)
             for line in output.splitlines():
                 # can have 1, 2 or 3 elements (commit, parent1, parent2)
@@ -601,7 +609,8 @@ class GitClient(VcsClientBase):
         commit. Else it would eventually be deleted.
 
         :param version: SHA IDs (if partial, caller is responsible for mismatch)
-        :param mask_self: whether to consider direct references to this commit (rather than only references on descendants) as well
+        :param mask_self: whether to consider direct references to this commit
+            (rather than only references on descendants) as well
         :param fetch: whether fetch should be done first for remote refs
         :returns: True if version is not recursively referenced by a branch or tag
         :raises: GitError if git fetch fails
@@ -650,7 +659,7 @@ class GitClient(VcsClientBase):
             os.remove(basepath + '.tar')
         return True
 
-    def _do_fetch(self):
+    def _do_fetch(self, timeout=None):
         """
         calls git fetch
         :raises: GitError when call fails
@@ -659,17 +668,21 @@ class GitClient(VcsClientBase):
         value1, _, _ = run_shell_command(cmd,
                                          cwd=self._path,
                                          shell=True,
+                                         no_filter=True,
+                                         timeout=timeout,
                                          show_stdout=True)
-        ## git fetch --tags ONLY fetches new tags and commits used, no other commits!
+        # git fetch --tags ONLY fetches new tags and commits used, no other commits!
         cmd = "git fetch --tags"
         value2, _, _ = run_shell_command(cmd,
                                          cwd=self._path,
                                          shell=True,
+                                         no_filter=True,
+                                         timeout=timeout,
                                          show_stdout=True)
         if value1 != 0 or value2 != 0:
             raise GitError('git fetch failed')
 
-    def _do_fast_forward(self, fetch=True, branch_parent=None, verbose=False):
+    def _do_fast_forward(self, branch_parent, fetch=True, verbose=False):
         """Execute git fetch if necessary, and if we can fast-foward,
         do so to the last fetched version using git rebase.
 
@@ -694,6 +707,7 @@ class GitClient(VcsClientBase):
                                       parent_version,
                                       fetch=False):
                 return True
+            print("Cannot fast-forward, local repository and remote '%s' have diverged." % branch_parent)
             return False
         if verbose:
             print("Rebasing repository")
@@ -751,5 +765,5 @@ class GitClient(VcsClientBase):
             raise GitError('Git Checkout failed')
 
 
-#Backwards compatibility
+# Backwards compatibility
 GITClient = GitClient
