@@ -93,18 +93,19 @@ class SvnClient(VcsClientBase):
          or None if it cannot be determined
         """
         if self.detect_presence():
-            #3305: parsing not robust to non-US locales
+            # 3305: parsing not robust to non-US locales
             cmd = 'svn info %s' % self._path
             _, output, _ = run_shell_command(cmd, shell=True)
             matches = [l for l in output.splitlines() if l.startswith('URL: ')]
             if matches:
                 return matches[0][5:]
 
-    def detect_presence(self):
-        return self.path_exists() and \
-            os.path.isdir(os.path.join(self.get_path(), '.svn'))
+    @staticmethod
+    def static_detect_presence(path):
+        return os.path.isdir(os.path.join(path, '.svn'))
 
-    def checkout(self, url, version='', verbose=False, shallow=False):
+    def checkout(self, url, version='', verbose=False,
+                 shallow=False, timeout=None):
         if url is None or url.strip() == '':
             raise ValueError('Invalid empty url : "%s"' % url)
         # Need to check as SVN 1.6.17 writes into directory even if not empty
@@ -128,7 +129,7 @@ class SvnClient(VcsClientBase):
             return False
         return True
 
-    def update(self, version=None, verbose=False):
+    def update(self, version=None, verbose=False, timeout=None):
         if not self.detect_presence():
             sys.stderr.write("Error: cannot update non-existing directory\n")
             return False
