@@ -184,6 +184,14 @@ class SvnClientTest(SvnClientTestSetups):
         client = SvnClient(self.remote_path)
         self.assertEquals('', client.get_status())
 
+    def test_get_environment_metadata(self):
+        # Verify that metadata is generated
+        directory = tempfile.mkdtemp()
+        self.directories['local'] = directory
+        local_path = os.path.join(directory, "local")
+        client = SvnClient(local_path)
+        self.assertTrue('version' in client.get_environment_metadata())
+
 
 class SvnClientLogTest(SvnClientTestSetups):
 
